(*
 * PasLibVlcPlayerUnit.pas - VCL component for libvlc 1.1.7
 *
 * See copyright notice below.
 *
 * Last modified: 2011.01.05
 *
 * author: Robert Jdrzejczyk
 * e-mail: robert@prog.olsztyn.pl
 *    www: http://www.prog.olsztyn.pl/paslibvlc
 *
 *******************************************************************************
 *
 * 2011.02.08 Robert Jdrzejczyk
 *
 * simple help for play YouTube video links.
 *
 * Now Play function detect YouTube link, and play it correctly.
 *
 * Requested by:
 *
 * Christian cf. Fillion <christian@rhesus.net>
 *
 *******************************************************************************
 *
 * 2011.01.05 Robert Jdrzejczyk
 *
 * add new functions:
 *
 *   GetVideoLenStr() - return video length as time string
 *   GetVideoPosStr() - return video position as time string
 *
 * Requested by:
 *
 * Edijs van Kole de McSnikovics <terminedijs@yahoo.com>
 *
 *******************************************************************************
 *
 * 2011.01.05 Robert Jdrzejczyk
 *
 * add new properties: PopupMenu, etc.
 *
 * Requested by:
 *
 * Edijs van Kole de McSnikovics <terminedijs@yahoo.com>
 *
 *******************************************************************************
 *
 * 2011.01.04 Robert Jdrzejczyk
 *
 * correct creation of VCL at runtime, error: control '' has no parent window
 *
 * This error will be found by:
 *
 * Edijs van Kole de McSnikovics <terminedijs@yahoo.com>
 *
 *******************************************************************************
 *
 * 2011.01.03 Robert Jdrzejczyk
 *
 * rename variable: FHideTitle to FShowTitle
 *
 *   if ShowTitle = TRUE  then SHOW title at begin of play
 *   if ShowTitle = FALSE then HIDE title at begin of play
 *
 *   default value: FALSE
 *
 *******************************************************************************
 *
 * 2010.12.08 Robert Jdrzejczyk
 *
 * Add support for version 1.1.5
 *
 * new properties:
 *
 *   HideTitle - if TRUE prevent display title at begin of play, default TRUE
 *   UseEvents - if TRUE then enable event propagation, default FALSE
 *
 *******************************************************************************
 *
 * 2010.10.01 Robert Jdrzejczyk
 *
 * Add support for unicode file names
 *
 *******************************************************************************
 *
 * 2010.07.22 Robert Jdrzejczyk
 *
 * Add support for set/get Audio Volume Level
 *
 *******************************************************************************
 *
 * 2010.09.02 Robert Jdrzejczyk
 *
 * Add support for version 1.1.4
 *
 *******************************************************************************
 *
 * 2010.07.14 David Nottage, davidnottage@gmail.com
 *
 * Change PChar to PAnsiChar
 * This help usage in Delphi 2010
 *)

(* This is file is part of PasLibVlcPlayerDemo program
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Any non-GPL usage of this software or parts of this software is strictly
 * forbidden.
 *
 * The "appropriate copyright message" mentioned in section 2c of the GPLv2
 * must read: "Code from FAAD2 is copyright (c) Nero AG, www.nero.com"
 *
 * Commercial non-GPL licensing of this software is possible.
 * please contact robert@prog.olsztyn.pl
 *
 *)

(*
 * libvlc is part of project VideoLAN
 *
 * Copyright (c) 1996-2010 VideoLAN Team
 *
 * For more information about VideoLAN
 *
 * please visit http://www.videolan.org
 *
 *)

unit PasLibVlcPlayerUnit;

interface

uses
  SysUtils, Classes, Controls, ExtCtrls, Messages, Graphics, PasLibVlcUnit;

type
  TPasLibVlcPlayerState = (
    plvPlayer_NothingSpecial,
    plvPlayer_Opening,
    plvPlayer_Buffering,
    plvPlayer_Playing,
    plvPlayer_Paused,
    plvPlayer_Stopped,
    plvPlayer_Ended,
    plvPlayer_Error);


type
  TNotifySeekableChanged = procedure(Sender : TObject; val      : Boolean) of object;
  TNotifyPausableChanged = procedure(Sender : TObject; val      : Boolean) of object;
  TNotifyTitleChanged    = procedure(Sender : TObject; title    : Integer) of object;
  TNotifySnapshotTaken   = procedure(Sender : TObject; fileName : string)  of object;
  TNotifyTimeChanged     = procedure(Sender : TObject; time     : Int64)   of object;
  TNotifyLengthChanged   = procedure(Sender : TObject; time     : Int64)   of object;
  TNotifyPositionChanged = procedure(Sender : TObject; position : Single)  of object;
  
type
  TPasLibVlcPlayer = class(TCustomPanel)
  private
    p_mi        : libvlc_media_player_t_ptr;
    p_md        : libvlc_media_t_ptr;
    p_mi_ev_mgr : libvlc_event_manager_t_ptr;

    //
    FError          : string;
    FMute           : Boolean;
    FShowTitle      : Boolean;
    FUseEvents      : Boolean;
    FSnapshotFormat : string;
    
    // events handlers
    FOnMediaPlayerMediaChanged     : TNotifyEvent;
    FOnMediaPlayerNothingSpecial   : TNotifyEvent;
    FOnMediaPlayerOpening          : TNotifyEvent;
    FOnMediaPlayerBuffering        : TNotifyEvent;
    FOnMediaPlayerPlaying          : TNotifyEvent;
    FOnMediaPlayerPaused           : TNotifyEvent;
    FOnMediaPlayerStopped          : TNotifyEvent;
    FOnMediaPlayerForward          : TNotifyEvent;
    FOnMediaPlayerBackward         : TNotifyEvent;
    FOnMediaPlayerEndReached       : TNotifyEvent;
    FOnMediaPlayerEncounteredError : TNotifyEvent;
    FOnMediaPlayerTimeChanged      : TNotifyTimeChanged;
    FOnMediaPlayerPositionChanged  : TNotifyPositionChanged;
    FOnMediaPlayerSeekableChanged  : TNotifySeekableChanged;
    FOnMediaPlayerPausableChanged  : TNotifyPausableChanged;
    FOnMediaPlayerTitleChanged     : TNotifyTitleChanged;
    FOnMediaPlayerSnapshotTaken    : TNotifySnapshotTaken;
    FOnMediaPlayerLengthChanged    : TNotifyLengthChanged;

    procedure SetUseEvents(aUseEvents: Boolean);
    procedure SetSnapshotFormat(aFormat: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure EventsEnable();
    procedure EventsDisable();

    procedure Play(mrl: WideString);
    procedure PlayYouTube(timeout: LongInt = 5000);
    
    procedure Pause();
    procedure Resume();
    function  IsPlay(): Boolean;

    function  GetState(): TPasLibVlcPlayerState;

    function  GetVideoWidth(): LongInt;
    function  GetVideoHeight(): LongInt;
    function  GetVideoScaleInPercent(): Single;
    procedure SetVideoScaleInPercent(newScaleInPercent: Single);
    function  GetAspectRatio(): string;
    procedure SetAspectRatio(newAspectRatio: string);
    
    function  GetVideoLenInMs(): Int64;
    function  GetVideoPosInMs(): Int64;
    procedure SetVideoPosInMs(newPos: Int64);
    function  GetVideoPosInPercent(): Single;
    procedure SetVideoPosInPercent(newPos: Single);
    function  GetVideoFps(): Single;

    function GetVideoLenStr(fmt: string = 'hh:mm:ss'): string;
    function GetVideoPosStr(fmt: string = 'hh:mm:ss'): string;

    function  GetVideoChapter(): Integer;
    procedure SetVideoChapter(newChapter: Integer);
    function  GetVideoChapterCount(): Integer;

    function  CanPlay(): Boolean;
    function  CanPause(): Boolean;
    function  CanSeek(): Boolean;

    function Snapshot(fileName: WideString): Boolean; overload;
    function Snapshot(fileName: WideString; width, height: LongWord): Boolean; overload;

    procedure NextFrame();

    function  GetAudioMute(): Boolean;
    procedure SetAudioMute(mute: Boolean);
    function  GetAudioVolume(): Integer;
    procedure SetAudioVolume(volumeLevel: Integer);
    
  published
    property Align;
    property Color  default clBlack;
    property Width  default 320;
    property Height default 240;

    property Constraints;
    property DragKind;
    property DragMode;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnAlignPosition;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseActivate;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
    
    property TitleShow : Boolean
      read    FShowTitle
      write   FShowTitle
      default FALSE;

    property UseEvents : Boolean
      read    FUseEvents
      write   SetUseEvents
      default FALSE;

    property SnapshotFormat : string
      read    FSnapshotFormat
      write   SetSnapshotFormat;

    property LastError: string
      read   FError
      write  FError;

    property OnMediaPlayerMediaChanged     : TNotifyEvent
      read  FOnMediaPlayerMediaChanged
      write FOnMediaPlayerMediaChanged;

    property OnMediaPlayerNothingSpecial   : TNotifyEvent
      read  FOnMediaPlayerNothingSpecial
      write FOnMediaPlayerNothingSpecial;

    property OnMediaPlayerOpening          : TNotifyEvent
      read  FOnMediaPlayerOpening
      write FOnMediaPlayerOpening;

    property OnMediaPlayerBuffering        : TNotifyEvent
      read  FOnMediaPlayerBuffering
      write FOnMediaPlayerBuffering;

    property OnMediaPlayerPlaying          : TNotifyEvent
      read  FOnMediaPlayerPlaying
      write FOnMediaPlayerPlaying;

    property OnMediaPlayerPaused           : TNotifyEvent
      read  FOnMediaPlayerPaused
      write FOnMediaPlayerPaused;

    property OnMediaPlayerStopped          : TNotifyEvent
      read  FOnMediaPlayerStopped
      write FOnMediaPlayerStopped;

    property OnMediaPlayerForward          : TNotifyEvent
      read  FOnMediaPlayerForward
      write FOnMediaPlayerForward;

    property OnMediaPlayerBackward         : TNotifyEvent
      read  FOnMediaPlayerBackward
      write FOnMediaPlayerBackward;

    property OnMediaPlayerEndReached       : TNotifyEvent
      read  FOnMediaPlayerEndReached
      write FOnMediaPlayerEndReached;

    property OnMediaPlayerEncounteredError : TNotifyEvent
      read  FOnMediaPlayerEncounteredError
      write FOnMediaPlayerEncounteredError;

    property OnMediaPlayerTimeChanged      : TNotifyTimeChanged
      read  FOnMediaPlayerTimeChanged
      write FOnMediaPlayerTimeChanged;

    property OnMediaPlayerPositionChanged  : TNotifyPositionChanged
      read  FOnMediaPlayerPositionChanged
      write FOnMediaPlayerPositionChanged;

    property OnMediaPlayerSeekableChanged  : TNotifySeekableChanged
      read  FOnMediaPlayerSeekableChanged
      write FOnMediaPlayerSeekableChanged;

    property OnMediaPlayerPausableChanged  : TNotifyPausableChanged
      read  FOnMediaPlayerPausableChanged
      write FOnMediaPlayerPausableChanged;

    property OnMediaPlayerTitleChanged     : TNotifyTitleChanged
      read  FOnMediaPlayerTitleChanged
      write FOnMediaPlayerTitleChanged;

    property OnMediaPlayerSnapshotTaken    : TNotifySnapshotTaken
      read  FOnMediaPlayerSnapshotTaken
      write FOnMediaPlayerSnapshotTaken;

    property OnMediaPlayerLengthChanged    : TNotifyLengthChanged
      read  FOnMediaPlayerLengthChanged
      write FOnMediaPlayerLengthChanged;
  end;

procedure Register;

implementation

{$R *.RES}
                    
uses
  Windows;
  
procedure Register;
begin
  RegisterComponents('PasLibVlc', [TPasLibVlcPlayer]);
end;

var
  p_li: libvlc_instance_t_ptr;

(*
 * lib_vlc_player_event_hdlr is route all events
 * beetwen lib_vlc and VLC Player components
 * in your appliaction.
 *
 * Please not add any code to this procedure, because
 * in some situations this procedure will be called
 * very offten.
 *
 *)

procedure lib_vlc_player_event_hdlr(p_event: libvlc_event_t_ptr; data: Pointer); cdecl;
var
  player: TPasLibVlcPlayer;
begin
  if (data = NIL) then exit;
  player := TPasLibVlcPlayer(data);
  
  if not Assigned(player) then exit;
  
  case p_event^.event_type of

    libvlc_MediaPlayerMediaChanged:
      if Assigned(player.OnMediaPlayerMediaChanged) then
        player.OnMediaPlayerMediaChanged(player);

    libvlc_MediaPlayerNothingSpecial:
      if Assigned(player.OnMediaPlayerNothingSpecial) then
        player.OnMediaPlayerNothingSpecial(player);

    libvlc_MediaPlayerOpening:
      if Assigned(player.OnMediaPlayerOpening) then
        player.OnMediaPlayerOpening(player);

    libvlc_MediaPlayerBuffering:
      if Assigned(player.OnMediaPlayerBuffering) then
        player.OnMediaPlayerBuffering(player);

    libvlc_MediaPlayerPlaying:
      if Assigned(player.OnMediaPlayerPlaying) then
        player.OnMediaPlayerPlaying(player);

    libvlc_MediaPlayerPaused:
      if Assigned(player.OnMediaPlayerPaused) then
        player.OnMediaPlayerPaused(player);

    libvlc_MediaPlayerStopped:
      if Assigned(player.OnMediaPlayerStopped) then
        player.OnMediaPlayerStopped(player);

    libvlc_MediaPlayerForward:
      if Assigned(player.OnMediaPlayerForward) then
        player.OnMediaPlayerForward(player);

    libvlc_MediaPlayerBackward:
      if Assigned(player.OnMediaPlayerBackward) then
        player.OnMediaPlayerBackward(player);

    libvlc_MediaPlayerEndReached:
      if Assigned(player.OnMediaPlayerEndReached) then
        player.OnMediaPlayerEndReached(player);

    libvlc_MediaPlayerEncounteredError:
      if Assigned(player.OnMediaPlayerEncounteredError) then
        player.OnMediaPlayerEncounteredError(player);

    libvlc_MediaPlayerTimeChanged:
      if Assigned(player.OnMediaPlayerTimeChanged) then
        player.OnMediaPlayerTimeChanged(player,
          p_event^.media_player_time_changed.new_time);

    libvlc_MediaPlayerPositionChanged:
      if Assigned(player.OnMediaPlayerPositionChanged) then
        player.OnMediaPlayerPositionChanged(player,
          p_event^.media_player_position_changed.new_position * 100);

    libvlc_MediaPlayerSeekableChanged:
      if Assigned(player.OnMediaPlayerSeekableChanged) then
        player.OnMediaPlayerSeekableChanged(player,
          p_event^.media_player_seekable_changed.new_seekable <> 0);

    libvlc_MediaPlayerPausableChanged:
      if Assigned(player.OnMediaPlayerPausableChanged) then
        player.OnMediaPlayerPausableChanged(player,
          p_event^.media_player_pausable_changed.new_pausable <> 0);

    libvlc_MediaPlayerTitleChanged:
      if Assigned(player.OnMediaPlayerTitleChanged) then
        player.OnMediaPlayerTitleChanged(player,
          p_event^.media_player_title_changed.new_title);

    libvlc_MediaPlayerSnapshotTaken:
      if Assigned(player.OnMediaPlayerSnapshotTaken) then
        player.OnMediaPlayerSnapshotTaken(player,
          p_event^.media_player_snapshot_taken.psz_filename);

    libvlc_MediaPlayerLengthChanged:
      if Assigned(player.OnMediaPlayerLengthChanged) then
        player.OnMediaPlayerLengthChanged(player,
          p_event^.media_player_length_changed.new_length);
  end;
end;

constructor TPasLibVlcPlayer.Create(AOwner: TComponent);
var
  args: array[0..1] of PAnsiChar;
begin
  inherited Create(AOwner);

  Color      := clBlack;
  Width      := 320;
  Height     := 240;
  
  FShowTitle      := FALSE;
  FUseEvents      := FALSE;
  FSnapshotFormat := 'png';
  
  ParentBackground := False;

  Caption    := '';
  BevelOuter := bvNone;

  p_mi        := NIL;
  p_md        := NIL;
  p_mi_ev_mgr := NIL;
  FMute       := FALSE;

  if (csDesigning in ComponentState) then exit;

  if (p_li = NIL) then
  begin
    libvlc_dynamic_dll_init();
    if (libvlc_dynamic_dll_error <> '') then
    begin
      Caption := 'libvlc_dynamic_dll_error';
      exit;
    end;

    args[0] := PAnsiChar(libvlc_dynamic_dll_path);
    args[1] := NIL;
    p_li    := libvlc_new(1, @args);
    FError  := StrPas(libvlc_errmsg());
    if (FError <> '') then
    begin
      Caption := FError;
      exit;
    end;
  end
  else libvlc_retain(p_li);

  if (p_li <> NIL) then
  begin
    Caption := libvlc_get_version();
  end;

  // create player instance
  p_mi := libvlc_media_player_new(p_li);

  // no media to play
  p_md := NIL;

  // no events
  p_mi_ev_mgr := NIL;

  if FUseEvents then EventsEnable();
end;

destructor TPasLibVlcPlayer.Destroy;
begin

  EventsDisable();
  Sleep(100);
  
  FOnMediaPlayerMediaChanged     := NIL;
  FOnMediaPlayerNothingSpecial   := NIL;
  FOnMediaPlayerOpening          := NIL;
  FOnMediaPlayerBuffering        := NIL;
  FOnMediaPlayerPlaying          := NIL;
  FOnMediaPlayerPaused           := NIL;
  FOnMediaPlayerStopped          := NIL;
  FOnMediaPlayerForward          := NIL;
  FOnMediaPlayerBackward         := NIL;
  FOnMediaPlayerEndReached       := NIL;
  FOnMediaPlayerEncounteredError := NIL;
  FOnMediaPlayerTimeChanged      := NIL;
  FOnMediaPlayerPositionChanged  := NIL;
  FOnMediaPlayerSeekableChanged  := NIL;
  FOnMediaPlayerPausableChanged  := NIL;
  FOnMediaPlayerTitleChanged     := NIL;
  FOnMediaPlayerSnapshotTaken    := NIL;
  FOnMediaPlayerLengthChanged    := NIL;
  
  if (p_mi <> NIL) then
  begin
    libvlc_media_player_release(p_mi);
  end;

  if (Assigned(libvlc_release)) then
  begin
    libvlc_release(p_li);
  end;
  
  inherited Destroy;
end;

procedure TPasLibVlcPlayer.SetUseEvents(aUseEvents: Boolean);
begin
  if (FUseEvents <> aUseEvents) then
  begin
    FUseEvents := aUseEvents;
    if FUseEvents then EventsEnable()
    else EventsDisable();
  end;
end;

procedure TPasLibVlcPlayer.SetSnapshotFormat(aFormat: string);
begin
  FSnapshotFormat := 'png';
  aFormat := AnsiLowerCase(aFormat);
  if ((aFormat = 'png') or (aFormat = 'jpg')) then
  begin
    FSnapshotFormat := aFormat;
  end;
end;

procedure TPasLibVlcPlayer.EventsEnable();
begin
  if Assigned(p_mi_ev_mgr) then exit;
  
  // get event manager
  
  p_mi_ev_mgr := libvlc_media_player_event_manager(p_mi);

  // register event handler

  if Assigned(p_mi_ev_mgr) then
  begin
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerMediaChanged,     lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerNothingSpecial,   lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerOpening,          lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerBuffering,        lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerPlaying,          lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerPaused,           lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerStopped,          lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerForward,          lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerBackward,         lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerEndReached,       lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerEncounteredError, lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerTimeChanged,      lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerPositionChanged,  lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerSeekableChanged,  lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerPausableChanged,  lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerTitleChanged,     lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerSnapshotTaken,    lib_vlc_player_event_hdlr, SELF);
    libvlc_event_attach(p_mi_ev_mgr, libvlc_MediaPlayerLengthChanged,    lib_vlc_player_event_hdlr, SELF);
  end;
end;

procedure TPasLibVlcPlayer.EventsDisable();
begin
  // unregister event handler

  if Assigned(p_mi_ev_mgr) then
  begin
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerMediaChanged,     lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerNothingSpecial,   lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerOpening,          lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerBuffering,        lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerPlaying,          lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerPaused,           lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerStopped,          lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerForward,          lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerBackward,         lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerEndReached,       lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerEncounteredError, lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerTimeChanged,      lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerPositionChanged,  lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerSeekableChanged,  lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerPausableChanged,  lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerTitleChanged,     lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerSnapshotTaken,    lib_vlc_player_event_hdlr, SELF);
    libvlc_event_detach(p_mi_ev_mgr, libvlc_MediaPlayerLengthChanged,    lib_vlc_player_event_hdlr, SELF);
  end;

  p_mi_ev_mgr := NIL;
end;

(*
 * mrl - media resource locator
 *
 * This can be file: c:\movie.avi
 *              ulr: http://host/movie.avi
 *              rtp: rstp://host/movie
 *)

procedure TPasLibVlcPlayer.Play(mrl: WideString);
begin
  // create new media
  if FileExists(mrl) then
  begin
    p_md := libvlc_media_new_path(p_li, PAnsiChar(System.UTF8Encode(mrl)));
  end
  else
  begin
    p_md := libvlc_media_new_location(p_li, PAnsiChar(System.UTF8Encode(mrl)));
  end;

  // assign self as view window
  libvlc_media_player_set_hwnd(p_mi, SELF.Handle);

  // assign media to player
  libvlc_media_player_set_media(p_mi, p_md);

  if FShowTitle then
  begin
    libvlc_media_add_option(p_md, 'video-title-show');
  end
  else
  begin
    libvlc_media_add_option(p_md, 'no-video-title-show');
  end;

  if (FSnapshotFormat <> '') then
  begin
    libvlc_media_add_option(p_md, PAnsiChar('snapshot-format=' + FSnapshotFormat));
    libvlc_media_add_option(p_md, 'no-snapshot-preview');
  end;

  //libvlc_media_add_option(p_md, 'no-video-on-top');
  //libvlc_media_add_option(p_md, 'no-overlay');

  // play
  libvlc_media_player_play(p_mi);
  
  // enable handling mouse events by vlc
  // libvlc_video_set_mouse_input(p_mi, 1);

  // help for play youtube link
  if (Pos('www.youtube.com', AnsiLowerCase(mrl)) > 0) then PlayYouTube();

  // release media
  if (p_md <> NIL) then
  begin
    libvlc_media_release(p_md);
    p_md := NIL;
  end;
end;

procedure TPasLibVlcPlayer.PlayYouTube(timeout: LongInt = 5000);
var
  p_ml: libvlc_media_list_t_ptr;
  sub_p_md: libvlc_media_t_ptr;
begin

  while ((GetState() <> plvPlayer_Ended) and (timeout > 0)) do
  begin
    Sleep(100);
    Dec(timeout, 0);
  end;

  p_ml := libvlc_media_subitems(p_md);
  if Assigned(p_ml) then
  begin
    libvlc_media_list_lock(p_ml);
    sub_p_md := libvlc_media_list_item_at_index(p_ml, 0);
    libvlc_media_list_unlock(p_ml);
    if Assigned(sub_p_md) then
    begin
      libvlc_media_player_set_media(p_mi, sub_p_md);
      libvlc_media_player_play(p_mi);
    end;
    libvlc_media_list_release(p_ml);
  end;
end;

procedure TPasLibVlcPlayer.Pause();
begin
  if (p_mi = NIL) then exit;

  if (GetState() = plvPlayer_Playing) then
  begin
    libvlc_media_player_pause(p_mi);
  end;
end;

procedure TPasLibVlcPlayer.Resume();
begin
  if (p_mi = NIL) then exit;

  if (GetState() = plvPlayer_Paused)  then
  begin
    libvlc_media_player_play(p_mi);
  end;
end;

function TPasLibVlcPlayer.GetState(): TPasLibVlcPlayerState;
begin
  Result := plvPlayer_NothingSpecial;
  
  if (p_mi = NIL) then exit;
  
  case libvlc_media_player_get_state(p_mi) of
    libvlc_NothingSpecial: Result := plvPlayer_NothingSpecial;
    libvlc_Opening:        Result := plvPlayer_Opening;
    libvlc_Buffering:      Result := plvPlayer_Buffering;
    libvlc_Playing:        Result := plvPlayer_Playing;
    libvlc_Paused:         Result := plvPlayer_Paused;
    libvlc_Stopped:        Result := plvPlayer_Stopped;
    libvlc_Ended:          Result := plvPlayer_Ended;
    libvlc_Error:          Result := plvPlayer_Error;
  end;
end;

(*
 * Get current video width in pixels
 * If autoscale (scale = 0) then return original video width
 * If not autoscale (scale = xxx) then return video width * scale
 *)

function TPasLibVlcPlayer.GetVideoWidth(): LongInt;
var
  px, py: LongWord;
begin
  Result := -1;
  if (p_mi = NIL) then exit;
  if (libvlc_video_get_size(p_mi, 0, px, py) = 0) then Result := px;
end;

(*
 * Get current video height in pixels
 * If autoscale (scale = 0) then return original video height
 * If not autoscale (scale = xxx) then return video height * scale
 *)

function TPasLibVlcPlayer.GetVideoHeight(): LongInt;
var
  px, py: LongWord;
begin
  Result := -1;
  if (p_mi = NIL) then exit;
  if (libvlc_video_get_size(p_mi, 0, px, py) = 0) then Result := py;
end;

function TPasLibVlcPlayer.IsPlay(): Boolean;
begin
  Result := (GetState() = plvPlayer_Playing);
end;

(*
 * Get current video scale
 * I scale this by 100 (lib vlc return this im range 0..1)
 * If autoscale is on then return 0 else return actual scale
 *
 *)

function TPasLibVlcPlayer.GetVideoScaleInPercent(): Single;
begin
  Result := -1;
  if (p_mi = NIL) then exit;
  Result := libvlc_video_get_scale(p_mi) * 100;
end;

(*
 * Set current video scale
 * I scale this by 100 (lib vlc return this im range 0..1)
 * If return 0 then autoscale is on
 *
 * Now not working, because bug in vlc
 * posted to:
 *              http://trac.videolan.org/vlc/ticket/3812
 *
 * I correct this bug in my sources, then recoompile VLC
 * all working ok.
 *)

procedure TPasLibVlcPlayer.SetVideoScaleInPercent(newScaleInPercent: Single);
begin
  if (p_mi = NIL) then exit;
  libvlc_video_set_scale(p_mi, newScaleInPercent / 100);
end;

function TPasLibVlcPlayer.GetAspectRatio(): string;
begin
  Result := '';
  if (p_mi = NIL) then exit;
  Result := libvlc_video_get_aspect_ratio(p_mi);
end;

procedure TPasLibVlcPlayer.SetAspectRatio(newAspectRatio: string);
begin
  if (p_mi = NIL) then exit;
  libvlc_video_set_aspect_ratio(p_mi, PAnsiChar(newAspectRatio));
end;

(*
 * Return video time length in miliseconds
 *)
 
function TPasLibVlcPlayer.GetVideoLenInMs(): Int64;
begin
  Result := -1;
  if (p_mi = NIL) then exit;
  if IsPlay() then
    Result := libvlc_media_player_get_length(p_mi);
end;

function w2s(w: word): string;
begin
  if (w < 10)  then Result := '0' + IntToStr(w)
  else              Result := IntToStr(w);
end;

function t2s(ttt: Int64; fmt: string): string;
var
  dd, hh, mm, ss, ms: word;
begin
  dd := 0;
  hh := 0;
  mm := 0;
  ss := 0;
  ms := 0;
  if (ttt > 0) then
  begin
    ms := ttt mod 1000; ttt := ttt div 1000;
    ss := ttt mod 60;   ttt := ttt div 60;
    mm := ttt mod 60;   ttt := ttt div 60;
    hh := ttt mod 24;   ttt := ttt div 24;
    dd := ttt;
  end;
  Result := fmt;
  Result := StringReplace(Result, 'dd',  w2s(dd), [rfReplaceAll, rfIgnoreCase]);
  Result := StringReplace(Result, 'hh',  w2s(hh), [rfReplaceAll, rfIgnoreCase]);
  Result := StringReplace(Result, 'mm',  w2s(mm), [rfReplaceAll, rfIgnoreCase]);
  Result := StringReplace(Result, 'ss',  w2s(ss), [rfReplaceAll, rfIgnoreCase]);
  Result := StringReplace(Result, 'ms',  w2s(ms), [rfReplaceAll, rfIgnoreCase]);
end;

(*
 * Return current video time length as string hh:mm:ss
 *)

function TPasLibVlcPlayer.GetVideoLenStr(fmt: string = 'hh:mm:ss'): string;
begin
  Result := t2s(GetVideoLenInMs(), fmt);
end;

(*
 * Return current video time position in miliseconds
 *)

function TPasLibVlcPlayer.GetVideoPosInMs(): Int64;
begin
  Result := -1;
  if (p_mi = NIL) then exit;
  Result := libvlc_media_player_get_time(p_mi);
end;

(*
 * Return current video time position as string hh:mm:ss
 *)

function TPasLibVlcPlayer.GetVideoPosStr(fmt: string = 'hh:mm:ss'): string;
begin
  Result := t2s(GetVideoPosInMs(), fmt);
end;

(*
 * Set current video time position in miliseconds
 * Not working for all media
 *)

procedure TPasLibVlcPlayer.SetVideoPosInMs(newPos: Int64);
begin
  if (p_mi = NIL) then exit;
  libvlc_media_player_set_time(p_mi, newPos);
end;

(*
 * Return current video position where 0 - start, 100 - end
 * I scale this by 100 (lib vlc return this im range 0..1)
 *)

function TPasLibVlcPlayer.GetVideoPosInPercent(): Single;
begin
  Result := -1;
  if (p_mi = NIL) then exit;
  Result := libvlc_media_player_get_position(p_mi) * 100;
end;

(*
 * Set current video position where 0 - start, 100 - end
 * I scale this by 100 (lib vlc return this im range 0..1)
 * Not working for all media
 *)

procedure TPasLibVlcPlayer.SetVideoPosInPercent(newPos: Single);
begin
  if (p_mi = NIL) then exit;
  libvlc_media_player_set_position(p_mi, newPos / 100);
end;

(*
 * Return frames per second
 *)

function TPasLibVlcPlayer.GetVideoFps(): Single;
begin
  Result := -1;
  if (p_mi = NIL) then exit;
  Result := libvlc_media_player_get_fps(p_mi);
end;

(*
 * Return chapter if current video has chapters
 *)

function TPasLibVlcPlayer.GetVideoChapter(): Integer;
begin
  Result := -1;
  if (p_mi = NIL) then exit;
  Result := libvlc_media_player_get_chapter(p_mi);
end;

(*
 * Go to the chapter if current video has chapters
 *)

procedure TPasLibVlcPlayer.SetVideoChapter(newChapter: Integer);
begin
  if (p_mi = NIL) then exit;
  libvlc_media_player_set_chapter(p_mi, newChapter);
end;

(*
 * Return chapter count if current video has chapters
 *)

function TPasLibVlcPlayer.GetVideoChapterCount(): Integer;
begin
  Result := -1;
  if (p_mi = NIL) then exit;
  Result := libvlc_media_player_get_chapter_count(p_mi);
end;

(*
 * Return true if player can play
 *)

function TPasLibVlcPlayer.CanPlay(): Boolean;
begin
  Result := FALSE;
  if (p_mi = NIL) then exit;
  Result := (libvlc_media_player_will_play(p_mi) > 0);
end;

(*
 * Return true if player can pause
 *)

function TPasLibVlcPlayer.CanPause(): Boolean;
begin
  Result := FALSE;
  if (p_mi = NIL) then exit;
  Result := (libvlc_media_player_can_pause(p_mi) > 0);
end;

(*
 * Return true if player can seek (set time or percent position)
 *)

function TPasLibVlcPlayer.CanSeek(): Boolean;
begin
  Result := FALSE;
  if (p_mi = NIL) then exit;
  Result := (libvlc_media_player_is_seekable(p_mi) > 0);
end;

(*
 * Create snapshot of current video frame to specified fileName
 * The file is in PNG format
 *)
 
function TPasLibVlcPlayer.SnapShot(fileName: WideString): Boolean;
var
  i_width, i_height: LongWord;
begin
  Result := FALSE;
  if (p_mi = NIL) then exit;
  if (libvlc_video_get_size(p_mi, 0, i_width, i_height) <> 0) then exit;
  Result := (libvlc_video_take_snapshot(p_mi, 0, PAnsiChar(System.UTF8Encode(fileName)), i_width, i_height) = 0);
end;

(*
 * Create snapshot of current video frame
 * to specified fileName with size width x heght
 * The file is in PNG format
 *)

function TPasLibVlcPlayer.Snapshot(fileName: WideString; width, height: LongWord): Boolean;
begin
  Result := FALSE;
  if (p_mi = NIL) then exit;
  Result := (libvlc_video_take_snapshot(p_mi, 0, PAnsiChar(System.UTF8Encode(fileName)), width, height) = 0);
end;

procedure TPasLibVlcPlayer.NextFrame();
begin
  if (p_mi = NIL) then exit;
  libvlc_media_player_next_frame(p_mi);
end;

function TPasLibVlcPlayer.GetAudioMute(): Boolean;
begin
  Result := FMute;
end;

procedure TPasLibVlcPlayer.SetAudioMute(mute: Boolean);
begin
  if (p_mi = NIL) then exit;
  if mute then libvlc_audio_set_mute(p_mi, 1)
  else         libvlc_audio_set_mute(p_mi, 0);
end;

function TPasLibVlcPlayer.GetAudioVolume(): Integer;
begin
  Result := -1;
  if (p_mi = NIL) then exit;
  Result := libvlc_audio_get_volume(p_mi);
end;

procedure TPasLibVlcPlayer.SetAudioVolume(volumeLevel: Integer);
begin
  if (p_mi = NIL) then exit;
  if (volumeLevel < 0) then exit;
  if (volumeLevel > 200) then exit;
  libvlc_audio_set_volume(p_mi, volumeLevel);
end;

initialization

   p_li := NIL; // library instance

finalization


end.
